    <link rel="stylesheet" href="<%= paths[:top] %>/test-unit.css" type="text/css" />
    <link rel="shortcut icon" href="<%= paths[:top] %>/favicon.ico" />
    <link rel="icon" href="<%= paths[:top] %>/favicon.png" />
    <title><%= title %> - test-unit</title>
