<div class="header">
  <div class="title">
    <a href="<%= paths[:top] %>/">
      <span class="title"><%= title %></span>
      <span class="title-separator">-</span>
      <span class="title-project">test-unit</span>
    </a>
  </div>
  <ul class="other-languages">
    <li id="other-language-japanese" lang="ja" xml:lang="ja"><a href="<%= paths[:top] %>/test-unit/ja/<%= paths[:current] %>">日本語</a></li>
  </ul>
  <ul class="menu">
    <li id="menu-reference"><a href="<%= paths[:top] %>/test-unit/en/">Reference Manual</a></li>
    <li id="menu-install"><a href="<%= paths[:top] %>/#test-unit-install">Install</a></li>
    <li id="menu-developer"><a href="<%= paths[:top] %>/#developer">Developers</a></li>
  </ul>
</div>

<div class="content">
