    </div>

    <div class="sponsors">
      <p id="sponsor-github">
        <a href="https://github.com/test-unit/">
          <img src="<%= paths[:top] %>/github-logo.png" width="100" height="45" border="0" alt="The test-unit project uses GitHub's git repositories." />
        </a>
      </p>
      <p id="sponsor-tango">
	<a href="http://tango.freedesktop.org/">
          <img width="120" height="53" border="0" alt="Tango Desktop Project's icons are used." src="<%= paths[:top] %>/tango-logo.png" />
        </a>
      </p>
      <p id="sponsor-famfamfam">
	<a href="http://www.famfamfam.com/">
          <img width="120" height="18" border="0" alt="famfamfam.com's flags are used." src="<%= paths[:top] %>/famfamfam-logo.png" />
        </a>
      </p>
      <p id="sponsor-famfamfam">
	<a href="http://www.famfamfam.com/">
          <img width="120" height="18" border="0" alt="famfamfam.com's flags are used." src="<%= paths[:top] %>/famfamfam-logo.png" />
        </a>
      </p>

<!-- Piwik -->
<a href="http://piwik.org" title="Web analytics" onclick="window.open(this.href);return(false);">
<script type="text/javascript">
var pkBaseURL = (("https:" == document.location.protocol) ? "https://www.clear-code.com/piwik/" : "http://www.clear-code.com/piwik/");
document.write(unescape("%3Cscript src='" + pkBaseURL + "piwik.js' type='text/javascript'%3E%3C/script%3E"));
</script><script type="text/javascript">
piwik_action_name = '';
piwik_idsite = 2;
piwik_url = pkBaseURL + "piwik.php";
piwik_log(piwik_action_name, piwik_idsite, piwik_url);
</script>
<object><noscript><p>Web analytics <img src="http://www.clear-code.com/piwik/piwik.php?idsite=2" style="border:0" alt=""/></p></noscript></object></a>
<!-- End Piwik Tag -->
    </div>
