<div class="header">
  <div class="title">
    <a href="<%= paths[:top] %>/ja/index.html">
      <span class="title"><%= title %></span>
      <span class="title-separator">-</span>
      <span class="title-project">test-unit</span>
    </a>
  </div>
  <ul class="other-languages">
    <li id="other-language-english" lang="en" xml:lang="en"><a href="<%= paths[:top] %>/test-unit/en/<%= paths[:current] %>">English</a></li>
  </ul>
  <ul class="menu">
    <li id="menu-reference"><a href="<%= paths[:top] %>/test-unit/ja/">リファレンスマニュアル</a></li>
    <li id="menu-install"><a href="<%= paths[:top] %>/ja/index.html#test-unit-install">インストール</a></li>
    <li id="menu-developer"><a href="<%= paths[:top] %>/ja/index.html#developer">開発者向け情報</a></li>
  </ul>
</div>

<div class="content">
